import defu from 'defu'

const axios = require('axios')
const bodyParser = require('body-parser')

function assignDefaults (strategy, defaults) {
  Object.assign(strategy, defu(strategy, defaults))
}

function addAuthorize (strategy) {
  // Get clientSecret, clientId, endpoints.token and audience
  const clientSecret = strategy.clientSecret
  const clientID = strategy.clientId
  const tokenEndpoint = strategy.endpoints.token
  const audience = strategy.audience

  // IMPORTANT: remove clientSecret from generated bundle
  delete strategy.clientSecret

  // Endpoint
  const endpoint = `/_auth/oauth/${strategy._name}/authorize`

  // Set response_type to code
  strategy.responseType = 'code'

  // Form data parser
  const formMiddleware = bodyParser.urlencoded({ extended: true })

  // Register endpoint
  this.options.serverMiddleware.unshift({
    path: endpoint,
    handler: (req, res, next) => {
      if (req.method !== 'POST') {
        return next()
      }

      formMiddleware(req, res, () => {
        const {
          code,
          redirect_uri: redirectUri = strategy.redirectUri,
          response_type: responseType = strategy.responseType,
          grant_type: grantType = strategy.grantType
        } = req.body

        if (!code) {
          return next()
        }

        axios
          .request({
            method: 'post',
            url: tokenEndpoint,
            data: {
              client_id: clientID,
              client_secret: clientSecret,
              grant_type: grantType,
              response_type: responseType,
              redirect_uri: redirectUri,
              audience: audience,
              code
            },
            headers: {
              Accept: 'application/json'
            }
          })
          .then(response => {
            res.end(JSON.stringify(response.data))
          })
          .catch(error => next(error))
      })
    }
  })
}

module.exports = {
  addAuthorize,
  assignDefaults
}
